@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: parse_aux.ml,v 1.13 2001/07/30 14:45:40 xleroy Exp $ *)
+(* $Id: parse_aux.ml,v 1.20 2004/07/08 09:50:40 xleroy Exp $ *)
 
 (* Auxiliary functions for parsing *)
 
@@ -95,18 +95,12 @@ let make_bigarray ty =
       extract_spine dims ty'
   | ty ->
       (List.rev dims, ty) in
-  let rec extract_unique = function
-    Type_array(attr, ty) -> attr.maybe_null
-  | Type_pointer(kind, ty) -> kind = Unique
-  | Type_const((Type_pointer(_,_) | Type_array(_,_)) as ty') ->
-      extract_unique ty'
-  | _ -> false in
   let (dims, ty_tail) = extract_spine [] ty in
   match ty_tail with
     Type_int(_,_) | Type_float | Type_double 
   | Type_const(Type_int(_,_) | Type_float | Type_double) ->
       Type_bigarray({dims = dims; fortran_layout = false; malloced = false;
-                     bigarray_maybe_null = extract_unique ty},
+                     bigarray_maybe_null = false},
                     ty_tail)
   | _ ->
       eprintf "%t: Warning: bigarray attribute applied to type `%a', ignored\n"
@@ -133,7 +127,14 @@ let rec apply_type_attribute ty attr =
   | (("unique", _), Type_pointer(attr, ty_elt)) ->
       Type_pointer(Unique, ty_elt)
   | (("unique", _), Type_array(attr, ty_elt)) ->
-      Type_array({attr with maybe_null = true}, ty_elt)
+      begin match attr.bound with
+        None ->
+          Type_array({attr with maybe_null = true}, ty_elt)
+      | Some _ ->
+          eprintf "%t: Warning: `unique' attribute not applicable to array \
+                     of fixed size, ignored\n" print_location;
+          Type_array(attr, ty_elt)
+      end
   | (("unique", _), Type_bigarray(attr, ty_elt)) ->
       Type_bigarray({attr with bigarray_maybe_null = true}, ty_elt)
   | (("ptr", _), Type_pointer(attr, ty_elt)) ->
@@ -143,9 +144,7 @@ let rec apply_type_attribute ty attr =
   | (("string", _), Type_array(attr, ty_elt)) ->
       Type_array({attr with is_string = true}, ty_elt)
   | (("string", _), Type_pointer(attr, ty_elt)) ->
-      let attr' = {no_bounds with is_string = true;
-                                  maybe_null = (attr = Unique)} in
-      Type_array(attr', ty_elt)
+      Type_array({no_bounds with is_string = true}, ty_elt)
   | (("null_terminated", _), Type_array(attr, ty_elt))->
       Type_array({attr with null_terminated = true}, ty_elt)
   | (("null_terminated", _), Type_pointer(attr, ty_elt)) ->
@@ -222,7 +221,11 @@ let make_param attrs tybase decl =
   merge_attributes None ty attrs
 
 let make_fun_declaration attrs ty_res name params quotes =
-  let call = ref None and dealloc = ref None in
+  let truename = ref name
+  and mlname = ref None
+  and call = ref None
+  and dealloc = ref None
+  and blocking = ref false in
   let parse_quote (label, text) =
     match String.lowercase label with
       "call" -> call := Some text
@@ -231,9 +234,12 @@ let make_fun_declaration attrs ty_res name params quotes =
         eprintf "%t: Warning: quote type `%s' unknown, ignoring the quote.\n"
                 print_location label in
   List.iter parse_quote quotes;
-  let truename = ref name in
   let rec merge_attributes ty = function
       [] -> ty
+    | ("mlname", [Expr_ident s]) :: rem ->
+          mlname := Some s; merge_attributes ty rem
+    | ("blocking", _) :: rem ->
+          blocking := true; merge_attributes ty rem
     | (("callback" | "local"), _) :: rem ->
           merge_attributes ty rem
     | ("propget", _) :: rem ->
@@ -244,33 +250,39 @@ let make_fun_declaration attrs ty_res name params quotes =
           truename := "putref_" ^ name; merge_attributes ty rem
     | attr :: rem ->
           merge_attributes (apply_type_attribute ty attr) rem in
+  let ty_res' = merge_attributes ty_res attrs in
   { fun_name = !truename;
     fun_mod = "";
-    fun_res = merge_attributes ty_res attrs;
+    fun_res = ty_res';
     fun_params = params;
+    fun_mlname = !mlname;
     fun_call = !call;
-    fun_dealloc = !dealloc }
+    fun_dealloc = !dealloc;
+    fun_blocking = !blocking }
 
 let make_field attrs tybase decl =
-  let rec mlname default = function
-    [] -> default
-  | ("mlname", [Expr_ident s]) :: rem -> mlname s rem
-  | (_, _) :: rem -> mlname default rem in
-  let (name, ty) = decl tybase in
-  { field_name = name; field_mlname = mlname name attrs;
-    field_typ = apply_type_attributes ty attrs }
+  let rec merge_attributes name ty = function
+    [] ->
+      (name, ty)
+  | ("mlname", [Expr_ident s]) :: rem ->
+      merge_attributes s ty rem
+  | attr :: rem ->
+      merge_attributes name (apply_type_attribute ty attr) rem in
+  let (name, raw_ty) = decl tybase in
+  let (mlname, ty) = merge_attributes name raw_ty attrs in
+  { field_name = name; field_mlname = mlname; field_typ = ty }
 
 let make_fields attrs tybase decls =
   List.map (make_field attrs tybase) decls
 
-let make_discriminated_union name switch_name switch_type body =
+let make_discriminated_union name union_name switch_name switch_type body =
   let ty_union =
     Type_union({ud_name = ""; ud_mod = ""; ud_stamp = 0; ud_cases = body},
                {discriminant = Expr_ident switch_name}) in
   { sd_name = name; sd_mod = ""; sd_stamp = 0;
     sd_fields = [ {field_name = switch_name; field_mlname = switch_name;
                    field_typ = switch_type};
-                  {field_name = "u"; field_mlname = "u";
+                  {field_name = union_name; field_mlname = union_name;
                    field_typ = ty_union} ] }
 
 let type_names =
@@ -291,6 +303,12 @@ let make_typedef attrs tybase decls =
       merge_attributes ty {td with td_c2ml = Some f} rem
   | ("ml2c", [Expr_ident f]) :: rem ->
       merge_attributes ty {td with td_ml2c = Some f} rem
+  | ("finalize", [Expr_ident f]) :: rem ->
+      merge_attributes ty {td with td_finalize = Some f} rem
+  | ("compare", [Expr_ident f]) :: rem ->
+      merge_attributes ty {td with td_compare = Some f} rem
+  | ("hash", [Expr_ident f]) :: rem ->
+      merge_attributes ty {td with td_hash = Some f} rem
   | ("mltype", [Expr_ident f]) :: rem ->
       merge_attributes ty {td with td_mltype = Some f} rem
   | ("mltype", [Expr_string f]) :: rem ->
@@ -310,6 +328,7 @@ let make_typedef attrs tybase decls =
               td_type = Type_void; (* dummy *)
               td_abstract = false; td_mltype = None;
               td_c2ml = None; td_ml2c = None;
+              td_finalize = None; td_compare = None; td_hash = None;
               td_errorcode = false; td_errorcheck = None} in
     let (ty', td') = merge_attributes ty td attrs in
     {td' with td_type = ty'} in
@@ -331,6 +350,9 @@ let make_typedef attrs tybase decls =
           split_decls (decl :: past) rem in
   split_decls [] decls
 
+let make_const_decl attr ty name v =
+  { cd_name = name; cd_type = apply_type_attributes ty attr; cd_value = v }
+
 let update_int_default dfl arg =
   match arg with
     [Expr_ident "camlint"] -> dfl := Iunboxed
