@@ -10,17 +10,17 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: struct.mli,v 1.6 2000/08/19 11:04:58 xleroy Exp $ *)
+(* $Id: struct.mli,v 1.7 2002/01/16 09:42:03 xleroy Exp $ *)
 
 (* Marshaling for structs *)
 
 open Idltypes
 
 val struct_ml_to_c : 
-  (out_channel -> bool -> string -> idltype -> string -> string -> unit) ->
-    out_channel -> bool -> struct_decl -> string -> string -> unit
+  (out_channel -> bool -> Prefix.t -> idltype -> string -> string -> unit) ->
+    out_channel -> bool -> Prefix.t -> struct_decl -> string -> string -> unit
 val struct_c_to_ml : 
-  (out_channel -> string -> idltype -> string -> string -> unit) ->
-    out_channel -> struct_decl -> string -> string -> unit
+  (out_channel -> Prefix.t -> idltype -> string -> string -> unit) ->
+    out_channel -> Prefix.t -> struct_decl -> string -> string -> unit
 
 val remove_dependent_fields: field list -> field list
