@@ -10,15 +10,15 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: union.mli,v 1.5 2000/08/19 11:04:58 xleroy Exp $ *)
+(* $Id: union.mli,v 1.6 2002/01/16 09:42:04 xleroy Exp $ *)
 
 (* Marshalling for unions *)
 
 open Idltypes
 
 val union_ml_to_c : 
-  (out_channel -> bool -> string -> idltype -> string -> string -> unit) ->
-    out_channel -> bool -> union_decl -> string -> string -> string -> unit
+  (out_channel -> bool -> Prefix.t -> idltype -> string -> string -> unit) ->
+    out_channel -> bool -> Prefix.t -> union_decl -> string -> string -> string -> unit
 val union_c_to_ml : 
-  (out_channel -> string -> idltype -> string -> string -> unit) ->
-    out_channel -> union_decl -> string -> string -> string -> unit
+  (out_channel -> Prefix.t -> idltype -> string -> string -> unit) ->
+    out_channel -> Prefix.t -> union_decl -> string -> string -> string -> unit
